# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from codecs import open
import json
import yaml


class Converter(object):
    def __init__(self, input_filepath ='', output_filepath ='', locale='en'):
        super(Converter, self).__init__()
        self.input_filepath = input_filepath
        self.output_filepath = output_filepath
        self.locale = locale

    def convert(self):
        with open(self.input_filepath, 'r', encoding='utf-8') as f:
            content = yaml.load(f)

        self.ensure_openapi_3(content)

        api = API()
        api.parse(content)
        res = api.format()

        with open(self.output_filepath, 'w', encoding='utf-8') as out:
            out.write(res)
            
    def convert_content(self,content):  
        content = yaml.load(content)
        
        self.ensure_openapi_3(content)
        
        api = API()
        api.parse(content)
        res = api.format()
        return res
        
    def ensure_openapi_3(self, content):
        if not content.get('openapi', '').startswith('3.'):
            raise ValueError('Expected an OpenAPI 3 specification file')

        

class Field(object):
    def __init__(self, name):
        super(Field, self).__init__()
        self.name = name
        self.examples = {}

    def parse(self, seg, data, required=False):
        self.type = seg.get('type', '')
        self.desc = seg.get('description', '')
        self.required = required
        self.example_value = seg.get('example', '')
        required_fields = seg.get('required', [])
        if self.type == 'object':
            self.fields = []
            sorted_items = sorted(seg.get('properties', {}).items(), key=lambda t: t[0])
            for name, value in sorted_items:
                required = True if name in required_fields else False
                field = Field(name)
                field.parse(value, data, required)
                self.fields.append(field)
        elif self.type == 'array':
            value = seg.get('items')
            field = Field('')
            field.parse(value, data, False)
            self.field = field

        self.examples = seg.get('x-examples', {})

    def format(self, level):
        r = ''
        if self.type == 'object':
            if self.name:
                r += '|{level}{field}|{type}|{required}|{description}|\n'.format(
                    level='+' * level,
                    field=self.name,
                    type=self.type,
                    required=self.required,
                    description=self.desc
                )
                level += 1
            for field in self.fields:
                r += field.format(level)
        elif self.type == 'array':
            field = self.field
            r += '|{level}{field}|[{type}]|{required}|{description}|\n'.format(
                level='+' * level,
                field=self.name,
                type=field.type,
                required=self.required,
                description=self.desc
            )
            r += field.format(level + 1)
        else:
            if not self.name:
                return r

            r += '|{level}{field}|{type}|{required}|{description}|\n'.format(
                level='+' * level,
                field=self.name,
                type=self.type,
                required=self.required,
                description=self.desc
            )

        return r

    def example(self):
        if self.examples:
            for k, v in self.examples.items():
                return v

        if self.type == 'object':
            r = {}
            for field in self.fields:
                r[field.name] = field.example()
        elif self.type == 'array':
            field = self.field
            r = [field.example()]
        else:
            if self.example_value:
                r = self.example_value
            else:
                if self.type == "integer" or self.type == "number":
                    r = 0
                elif self.type == "boolean":
                    r = False
                else:
                    r = self.type

        return r


class ComponentSchema(object):
    def __init__(self, name):
        super(ComponentSchema, self).__init__()
        self.name = name

    def parse(self, seg, data):
        self.title = seg.get('title', '')
        self.type = seg.get('type', '')
        self.desc = seg.get('description', '')
        self.tags = seg.get('x-tags', '')

        field = Field('')
        field.parse(seg, data)
        self.field = field

    def format(self, level=0):
        r = '### {name}\n\n'.format(name=self.name)
        if self.desc:
            r += '{desc}\n\n'.format(desc=self.desc)

        r += '**Properties**\n\n'
        r += '|Field|Type|Required|Description|\n'
        r += '|---|---|---|---|\n'
        r += self.field.format(level)
        r += '\n'

        r += '**Example**\n\n'
        r += '```json\n'
        r += json.dumps(self.field.example(), indent=4) + '\n'
        r += '```\n'
        return r

    def example(self):
        r = self.field.example()
        return r


class Parameter(object):
    def __init__(self):
        pass

    def parse(self, seg, data):
        self.name = seg.get('name', '')
        self.type = seg.get('schema', {}).get('type', 'string')
        self.where = seg.get('in', '')
        self.required = seg.get('required', False)
        self.desc = seg.get('description', False)

    def format(self):
        r = '|{field}|{where}|{type}|{required}|{description}|\n'.format(
            field=self.name,
            where=self.where,
            type=self.type,
            required=self.required,
            description=self.desc
        )
        return r


class RequestBody(object):
    def __init__(self, name):
        self.name = name
        self.fmt = []
        self.field = None
        self.content_type = 'application/json'

    def parse(self, seg, data):
        self.desc = seg.get('description', '')
        content = seg.get('content', {})
        schema = content.get('application/json', {}).get('schema', {})
        if not schema:
            schema = content.get('multipart/form-data', {}).get('schema', {})
            if schema:
                self.content_type = 'multipart/form-data'

        if schema:
            self.fmt.append('application/json')
            field = Field('')
            field.parse(schema, data)
            self.field = field
        self.examples = content.get('application/json', {}).get('examples', {})

    def format(self):
        r = ''
        if not self.field:
            return r

        r += 'Body Type: ' + self.content_type + '\n\n'
        r += 'Body Parameter\n\n'
        r += '|Field|Type|Required|Description|\n'
        r += '|---|---|---|---|\n'
        r += self.field.format(level=0)
        r += '\n'
        return r

    def example(self):
        r = ''
        if not self.field and not self.examples:
            return r

        r += 'Body Example\n\n'
        r += '```json\n'
        if self.examples:
            for k, v in self.examples.items():
                r += json.dumps(v.get('value', {}), indent=4) + '\n'
                break
        else:
            r += json.dumps(self.field.example(), indent=4) + '\n'
        r += '```\n'
        return r


class Response(object):
    def __init__(self, name):
        self.name = name
        self.fmt = []
        self.field = None
        self.examples = {}
        self.content_type = 'application/json'

    def parse(self, seg, data):
        self.desc = seg.get('description', '')
        content = seg.get('content', {})
        schema = content.get('application/json', {}).get('schema', {})
        if not schema:
            schema = content.get('multipart/form-data', {}).get('schema', {})
            if schema:
                self.content_type = 'multipart/form-data'

        if schema:
            self.fmt.append('application/json')
            field = Field('')
            field.parse(schema, data)
            self.field = field
        self.examples = content.get('application/json', {}).get('examples', {})

    def format(self):
        r = ''
        if not self.field:
            return r

        r += 'Status Code {name}\n\n'.format(name=self.name)
        r += 'Body Type: ' + self.content_type + '\n\n'
        r += '|Field|Type|Required|Description|\n'
        r += '|---|---|---|---|\n'
        r += self.field.format(level=0)
        r += '\n'
        return r

    def example(self):
        r = ''
        if not self.field and not self.examples:
            return r

        r += '{name} Response\n\n'.format(name=self.name)
        r += '```json\n'
        if self.examples:
            for k, v in self.examples.items():
                r += json.dumps(v.get('value', {}), indent=4) + '\n'
                break
        else:
            r += json.dumps(self.field.example(), indent=4) + '\n'
        r += '```\n'
        return r


class Operation(object):
    def __init__(self, name):
        self.name = name
        self.parameters = []
        self.responses = []
        self.request_body = None

    def parse(self, seg, data):
        self.desc = seg.get('description', '')
        self.summary = seg.get('summary', '')
        self.id = seg.get('operationId', '')
        self.tags = seg.get('tags', [])
        for value in seg.get('parameters', []):
            parameter = Parameter()
            parameter.parse(value, data)
            self.parameters.append(parameter)

        sorted_items = sorted(seg.get('responses', {}).items(), key=lambda t: t[0])
        for name, value in sorted_items:
            response = Response(name)
            response.parse(value, data)
            self.responses.append(response)

        request_body = seg.get('requestBody', {})
        if request_body:
            self.request_body = RequestBody('')
            self.request_body.parse(request_body, data)

    def format(self):
        r = '#### {name}\n\n'.format(name=self.name.upper())
        if self.desc:
            r += '{desc}\n\n'.format(desc=self.desc)
        if self.parameters:
            r += '**Parameters**\n\n'
            r += '|Field|In|Type|Required|Description|\n'
            r += '|---|---|---|---|---|\n'
            for parameter in self.parameters:
                r += parameter.format()
            r += '\n'

        if self.request_body:
            r += '**Request Body**\n\n'
            r += self.request_body.format()
            r += self.request_body.example()

        if self.responses:
            r += '**Responses**\n\n'
            if self.responses:
                r += '|Status|Description|\n'
                r += '|---|---|\n'
                for response in self.responses:
                    r += '|{status}|{desc}|\n'.format(
                        status=response.name,
                        desc=response.desc
                    )
                r += '\n'

            r += '**Response Schema**\n\n'
            for response in self.responses:
                r += response.format()

            r += '**Response Example**\n\n'
            for response in self.responses:
                r += response.example()

        return r


class Path(object):
    def __init__(self, name):
        self.name = name
        self.operations = []
        self.parameters = []

    def parse(self, seg, data):
        sorted_items = sorted(seg.items(), key=lambda t: t[0])
        for name, value in sorted_items:
            if name == 'parameters':
                for v in value:
                    parameter = Parameter()
                    parameter.parse(v, data)
                    self.parameters.append(parameter)
                continue
            operation = Operation(name)
            operation.parse(value, data)
            self.operations.append(operation)

    def format(self):
        r = '### {name}\n\n'.format(name=self.name)
        for operation in self.operations:
            r += operation.format()
        return r


class Component(object):
    def __init__(self):
        self.schemas = []

    def parse(self, seg, data):
        sorted_items = sorted(seg.get('schemas', {}).items(), key=lambda t: t[0])
        for name, value in sorted_items:
            cs = ComponentSchema(name)
            cs.parse(value, data)
            self.schemas.append(cs)

    def format(self):
        r = '## Schemas\n\n'
        for schema in self.schemas:
            r += schema.format()
        return r


class Info():
    def __init__(self):
        self.license = {}
        self.contact = {}

    def parse(self, seg, data):
        self.title = seg.get('title', '')
        self.version = seg.get('version', '')
        self.description = seg.get('description', '')
        self.license = seg.get('license', {})
        self.contact = seg.get('contact', {})

    def format(self):
        r = '# {title}\n\n'.format(title=self.title)
        r += '{description}\n\n'.format(description=self.description)
        if self.contact:
            r += 'Contact:\n\n'
            r += 'Name: {name}\n\n'.format(name=self.contact.get('name', ''))
            r += 'Email: {email}\n\n'.format(email=self.contact.get('email', ''))
        if self.license:
            r += 'License:\n\n'
            r += 'Name: {name}\n\n'.format(name=self.license.get('name', ''))
            r += 'URL: {url}\n\n'.format(url=self.license.get('url', ''))

        return r


class API(object):
    def __init__(self):
        self.info = None
        self.servers = []
        self.paths = []
        self.components = None
        self.security = []
        self.tags = []

    def parse(self, data):
        self.info = Info()
        self.info.parse(data.get('info', {}), data)
        sorted_items = sorted(data.get('paths', {}).items(), key=lambda t: t[0])
        for name, value in sorted_items:
            path = Path(name)
            path.parse(value, data)
            self.paths.append(path)

        component = Component()
        component.parse(data['components'], data)
        self.components = component

    def format(self):
        r = self.info.format()
        if self.paths:
            r += '## API\n\n'
            for path in self.paths:
                r += path.format()
        if self.components:
            r += self.components.format()
        return r
    



# -*- coding: utf-8 -*-
'''
Created on 2021-1-10

@author: zhys513(254851907@qq.com)
'''

from opspipe.pipe.cv.data.conver.base import Base

class Voc(Base): 
    def __init__(self, data = None): 
        Base.__init__(self, data)  
        self.images = [] 
        self.annotations = []
        self.categories = []
    
    def read(self,path):  
        train_file='train_data.txt'
        labels_file='labels.txt'
        
        
    def converfrom(self,fun):
        self.data = fun.data
    
    def save(self,path):
        assert self.data == None
        print(path)


# -*- coding: utf-8 -*-
'''
Created on 2020-11-12

@author: zhys513(254851907@qq.com)
'''
# -*- coding: utf-8 -*-#
#-------------------------------------------------------------------------------
# Name:         table-cell-to-coco.py
# Author:       wdf
# Date:         2019/7/20
# IDE：         PyCharm 
# Parameters:
#     @param:
#     @param:
# Return： 
#       
# Description:  把左上角、右下角坐标转换为coco格式
# Usage：
#-------------------------------------------------------------------------------

import datetime
import json
from pathlib import Path
import re
from PIL import Image
import numpy as np
import progressbar  #用于Python的文本进度条库。文本进度条通常用于显示长时间运行的操作的进度，提供正在进行处理的可视化提示。
from multiprocessing import pool
from pycococreatortools import pycococreatortools

ROOT_DIR = Path('/media/tristan/Files/dataset/TableBank/table-cell')
IMAGE_DIR = ROOT_DIR / Path('images')
ANNOTATION_DIR = ROOT_DIR / Path('labels')

INFO = {
    "description": "TABLE-CELL Dataset",
    "url": "https://github.com/weidafeng",
    "version": "0.1.0",
    "year": 2019,
    "contributor": "DafengWei",
    "date_created": datetime.datetime.utcnow().isoformat(' ')  # 显示此刻时间，格式：'2019-04-30 02:17:49.040415'
}

LICENSES = [
    {
        "id": 1,
        "name": "Attribution-NonCommercial-ShareAlike License",
        "url": "http://creativecommons.org/licenses/by-nc-sa/2.0/"
    }
]

CATEGORIES = [
    {
        'id': 1,
        'name': 'cell',
        'supercategory': 'shape',
    },
    {
        'id': 2,
        'name': 'background',
        'supercategory': 'shape',
    }
    # {
    #     'id': 3,
    #     'name': 'ignore',
    #     'supercategory': 'shape',
    # }
]


# 获取 bounding-box， segmentation 信息
def get_info(content):
    # 输入content是list格式，
    # 获得bounding-box的坐标和内容, 以及segmentation信息：有序的四个点的坐标（bounding-box坐标）
    left, top  = float(content[1][0]), float(content[1][1])  # 左上角
    right, down = float(content[0][0]), float(content[0][1])  # 右下角

    height = down - top
    width = right-left
    #     word = content['word'] # 不考虑
    segmentation = [left,top, left + width, top, left + width, top + height, left, top + height] # 浮点形式
    return [left, top, width, height], [segmentation] # bounding-box信息， coco格式： x,y,w,h）；segmentation为[[1,2,3,4,5,6，7,8]]格式

def main():
    # coco lable文件（如training2017.json）需要存储的信息
    coco_output = {
        "info": INFO,
        "licenses": LICENSES,
        "categories": CATEGORIES,
        "images": [],
        "annotations": []
    }

    # 初始化id（以后依次加一）
    image_id = 1
    annotation_id = 1

    # 加载图片信息
    im_files = [f for f in IMAGE_DIR.iterdir()]
    im_files.sort(key=lambda f: f.stem,reverse=True)  # 以文件名排序，防止顺序错乱、数据和标签不对应
    # print("im-length:",len(im_files),"\n im_files：",im_files)

    # 加载annotation信息
    an_files = [f for f in ANNOTATION_DIR.iterdir()]
    an_files.sort(key=lambda f: f.stem,reverse=True)  # 以文件名排序，防止顺序错乱、数据和标签不对应
    # print("an-length:",len(an_files),"\n an_files：",an_files)

    assert len(an_files)==len(im_files), "图片数与lablel文件数目不匹配，请运行diff_two_folder.py，删除不匹配的文件" # 确保每个图片与label文件相匹配

    for im_file, an_file in zip(im_files, an_files):
        # 以coco格式，写入图片信息（id、图片名、图片大小）,其中id从1开始
        image = Image.open(im_file)
        im_info = pycococreatortools.create_image_info( image_id, im_file.name, image.size) # 图片信息
        coco_output['images'].append(im_info) # 存储图片信息（id、图片名、大小）
        myPool = pool.Pool(processes=16)  # 并行化处理

        # 开始处理label 信息
        annotation_info_list = []  # 存储单张图片的所有标注信息

        with open(an_file, 'r') as f:
            datas = json.load(f)
            for i in range(len(datas)):
                data = datas[i]
                # print(data)
                bounding_box = get_info(data)[0]
                segmentation = get_info(data)[1]  # 有序的四个点的坐标（bounding-box坐标）
                # print(bounding_box)
                # print(segmentation)

                class_id = 1  # label 数字形式

                # 显示日志
                print(bounding_box, segmentation)
                area = bounding_box[-1] * bounding_box[-2]  # 当前bounding-box的面积,宽×高
                # an_infos = pycococreatortools.mask_create_annotation_info(annotation_id=annotation_id, image_id=image_id, category_id=class_id, area=area, image_size=image.size, bounding_box=bounding_box,segmentation = segmentation)
                # annotation_info_list.append(an_infos)
                myPool.apply_async(func=pycococreatortools.mask_create_annotation_info,
                                    args= (annotation_id, image_id, category_id, area, image.size, bounding_box, segmentation),
                                    callbacks=annotation_info_list.append)
                annotation_id += 1

        myPool.close()
        myPool.join()
        # 上面得到单张图片的所有bounding-box信息，接下来每单张图片存储一次
        for annotation_info in annotation_info_list:
            if annotation_info is not None:
                coco_output['annotations'].append(annotation_info)
        image_id += 1

    # 保存成json格式
    print("保存annotations文件")
    output_json = Path(f'table_coco.json')
    with output_json.open('w', encoding='utf-8') as f:
        json.dump(coco_output, f)
    print("Annotations JSON file saved in：", str(output_json))

if __name__ == "__main__":
    main()


import cv2
import numpy as np
import os


def table_detection(img_path):
    img = cv2.imread(img_path)
    img_gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    (thresh, img_bin) = cv2.threshold(img_gray, 128, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)
    img_bin = cv2.bitwise_not(img_bin)

    kernel_length_v = (np.array(img_gray).shape[1])//120
    vertical_kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (1, kernel_length_v)) 
    im_temp1 = cv2.erode(img_bin, vertical_kernel, iterations=3)
    vertical_lines_img = cv2.dilate(im_temp1, vertical_kernel, iterations=3)

    kernel_length_h = (np.array(img_gray).shape[1])//40
    horizontal_kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (kernel_length_h, 1))
    im_temp2 = cv2.erode(img_bin, horizontal_kernel, iterations=3)
    horizontal_lines_img = cv2.dilate(im_temp2, horizontal_kernel, iterations=3)

    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (3, 3))
    table_segment = cv2.addWeighted(vertical_lines_img, 0.5, horizontal_lines_img, 0.5, 0.0)
    table_segment = cv2.erode(cv2.bitwise_not(table_segment), kernel, iterations=2)
    thresh, table_segment = cv2.threshold(table_segment, 0, 255, cv2.THRESH_OTSU)
   
    _, contours, hierarchy = cv2.findContours(table_segment, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
    count = 0
    for c in contours:
        x, y, w, h = cv2.boundingRect(c)
        if (w > 80 and h > 20) and w > 3 * h:
            count += 1
            cropped = img[y:y + h, x:x + w]
            
            if True:
                cv2.imwrite("./results/cropped/crop_" + str(count) + "__" + img_path.split('/')[-1], cropped)
        cv2.rectangle(img,(x, y),(x + w, y + h),(0, 255, 0), 2)

    if True:    
        cv2.imwrite("./results/table_detect/table_detect__" + img_path.split('/')[-1], table_segment)
        cv2.imwrite("./results/bb/bb__" + img_path.split('/')[-1], img)


{table_detection('./forms/' + i) for i in os.listdir('./forms/') if i.endswith('.png') or i.endswith('.PNG') or i.endswith('.jpg') or i.endswith('.JPG')}



# -*- coding: utf-8 -*-
'''
'''
 
import os, sys
from tqdm import tqdm
from loguru import logger
from theta.utils import load_json_file, split_train_eval_examples
from theta.modeling import LabeledText, load_ner_examples, load_ner_labeled_examples, save_ner_preds, show_ner_datainfo
 
#  if os.environ['NER_TYPE'] == 'span':
#      from theta.modeling.ner_span import load_model, get_args
#  else:
#      from theta.modeling.ner import load_model, get_args


import os
import numpy as np

best_model_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'outputs', 'latest', 'best')
base_model_path = '/opt/share/pretrained/pytorch/bert-base-chinese'

# read_txts(Path(dataset_dir,dataset_name,'labels.txt'))
ner_labels = [
    'game',
    'scene',
    'vx',
    'name',
    'company',
    'organization',
    'mobile',
    'movie',
    'email',
    'QQ',
    'government',
    'position',
    'book',
    'address'
]


# -------------------- Data --------------------

def clean_text(text):
    text = str(text).replace('\n', ' ')
    # if text:
    #    text = text.strip()
    # text = re.sub('\n', ' ', text)
    return text


def train_data_generator(train_file):
    lines = load_json_file(train_file)

    for i, x in enumerate(tqdm(lines)):
        guid = str(i)
        text = clean_text(x['originalText'])
        sl = LabeledText(guid, text)

        entities = x['entities']
        for entity in entities:
            start_pos = entity['start_pos']
            end_pos = entity['end_pos'] - 1
            category = entity['label_type']
            mention = entity['mention']

            # 矫正前后带空格
            ent = text[start_pos:end_pos + 1]
            if mention != ent:
                print('###', mention, ent)
            if len(ent.lstrip()) != len(ent):
                l = len(ent) - len(ent.lstrip())
                start_pos = start_pos + l
                print(ent, '--lstrip--', text[start_pos:end_pos + 1])

            if len(ent.rstrip()) != len(ent):
                l = len(ent) - len(ent.rstrip())
                end_pos = end_pos - l
                print(ent, '--rstrip--', text[start_pos:end_pos + 1])

            if category not in ner_labels:
                continue

            sl.add_entity(category, start_pos, end_pos)

        yield str(i), text, None, sl.entities


'''
def load_train_val_examples(args):

    allow_overlap = args.allow_overlap
    if args.num_augements > 0:
        allow_overlap = False

    lines = []
    for guid, text, _, entities in train_data_generator(args.train_file):
        sl = LabeledText(guid, text, entities)
        lines.append({'guid': guid, 'text': text, 'entities': entities})



    logger.info(f"Loaded {len(train_examples)} train examples, "
                f"{len(val_examples)} val examples.")
    return train_examples, val_examples 
'''


def load_train_val_examples(args):
    lines = []
    for guid, text, _, entities in train_data_generator(args.train_file):
        sl = LabeledText(guid, text, entities)
        lines.append({'guid': guid, 'text': text, 'entities': entities})

    allow_overlap = args.allow_overlap
    if args.num_augements > 0:
        allow_overlap = False

    train_base_examples = load_ner_labeled_examples(
        lines,
        ner_labels,
        seg_len=args.seg_len,
        seg_backoff=args.seg_backoff,
        num_augements=args.num_augements,
        allow_overlap=allow_overlap)

    train_examples, val_examples = train_base_examples, []
    # 如果训练集和验证集一样就自动切分
    if args.train_file == args.eval_file:
        train_examples, val_examples = split_train_eval_examples(
            train_base_examples,
            train_rate=args.train_rate,
            fold=args.fold,
            shuffle=False)
    else:
        lines = []
        for guid, text, _, entities in train_data_generator(args.eval_file):
            sl = LabeledText(guid, text, entities)
            lines.append({'guid': guid, 'text': text, 'entities': entities})

        val_examples = load_ner_labeled_examples(
            lines,
            ner_labels,
            seg_len=args.seg_len,
            seg_backoff=args.seg_backoff,
            num_augements=args.num_augements,
            allow_overlap=allow_overlap)

    logger.info(f"Loaded {len(train_examples)} train examples, "
                f"{len(val_examples)} val examples.")
    return train_examples, val_examples


def test_data_generator(test_file):
    lines = load_json_file(test_file)
    for i, s in enumerate(tqdm(lines)):
        i = clean_text(s['uid'])
        guid = str(i)
        text_a = clean_text(s['originalText'])

        yield guid, text_a, None, None


def load_test_examples(args):
    test_base_examples = load_ner_examples(test_data_generator,
                                           args.test_file,
                                           seg_len=args.seg_len,
                                           seg_backoff=args.seg_backoff)

    logger.info(f"Loaded {len(test_base_examples)} test examples.")
    return test_base_examples


class thetaNER():
    def __init__(self, dataset_name='ccf2020_business_ner', dataset_dir='datasets',
                 num_train_epochs=1,
                 train_max_seq_length=258,
                 seg_len=256,
                 seg_backoff=128, train_rate=0.9,
                 learning_rate=2e-5,
                 batch_size=4,
                 fold=0
                 ):

        self.num_train_epochs = num_train_epochs
        self.train_max_seq_length = train_max_seq_length
        self.seg_len = seg_len
        self.seg_backoff = seg_backoff
        self.dataset_dir = dataset_dir
        self.dataset_name = dataset_name
        self.train_rate = train_rate

        self.learning_rate = learning_rate
        self.batch_size = batch_size
        self.fold = fold
        self.model_path = base_model_path


    # print(self.get_params())
    def fit(self, parm):
       print('aaaa')

    def predict(self, X_new):
        return [],[]

    def get_params(self, deep=True):
        """Get parameters for this estimator.

        Parameters
        ----------
        deep : boolean, optional
            If True, will return the parameters for this estimator and
            contained subobjects that are estimators.

        Returns
        -------
        params : mapping of string to any
            Parameter names mapped to their values.
        """
        out = dict()
        for key in ['learning_rate', 'batch_size', 'fold']:  # 这里是所用超参数的list
            value = getattr(self, key, None)
            if deep and hasattr(value, 'get_params'):
                deep_items = value.get_params().items()
                out.update((key + '__' + k, val) for k, val in deep_items)
            out[key] = value
        return out

    def set_params(self, **params):
        """Set the parameters of this estimator.

        The method works on simple estimators as well as on nested objects
        (such as pipelines). The latter have parameters of the form
        ``<component>__<parameter>`` so that it's possible to update each
        component of a nested object.

        Returns
        -------
        self
        """
        if not params:
            # Simple optimization to gain speed (inspect is slow)
            return self
        valid_params = self.get_params(deep=True)

        for key, value in params.items():
            if key not in valid_params:
                raise ValueError('Invalid parameter %s for estimator %s. '
                                 'Check the list of available parameters '
                                 'with `estimator.get_params().keys()`.' %
                                 (key, self))
            setattr(self, key, value)
            valid_params[key] = value

        return self

    def score(self, X, y=None, sample_weight=None):


        return 111  # myloss_fun(y, self.predict(X), sample_weight=sample_weight)



if __name__ == '__main__':
    from sklearn.model_selection import RandomizedSearchCV

    param_grid = [
        {
         'fold':[1]
         },
    ]

    thetaNER = thetaNER()
    grid_search = RandomizedSearchCV(thetaNER, param_grid)
    grid_search.fit(np.arange(0,5))

    #返回最优的训练器
    print("Test set score:{:.2f}".format(grid_search.score(X=None)))
    print("Best parameters:{}".format(grid_search.best_params_))
    print("Best score on train set:{:.2f}".format(grid_search.best_score_))



# -*- coding: utf-8 -*-
'''
各种数据读取工具
Created on 2020-9-29
@author: zhys513(254851907@qq.com)
'''
import json, os,re
import fitz, zipfile
import shutil
from ..base.logger import logtime  

def clean_text(text):
    text = str(text)
    if text:
        text = text.strip()
        #  text = re.sub('\t', ' ', text)
    return text

def make_zip(source_dir, output_filename):
    '''打包目录为zip文件（未压缩）'''
    zipf = zipfile.ZipFile(output_filename, 'w')
    pre_len = len(os.path.dirname(source_dir))
    for parent, dirnames, filenames in os.walk(source_dir):
        for filename in filenames:
            pathfile = os.path.join(parent, filename)
            arcname = pathfile[pre_len:].strip(os.path.sep)   #相对路径
            zipf.write(pathfile, arcname)
    zipf.close()
 
 
def un_zip(file_name):  
    """unzip zip file"""  
    zip_file = zipfile.ZipFile(file_name)  
    if os.path.isdir(file_name + "_files"):  
        pass  
    else:  
        os.mkdir(file_name + "_files")  
    for names in zip_file.namelist():  
        zip_file.extract(names,file_name + "_files/")  
    zip_file.close()  


def read_txts(filename):
    '''读取单个txt文件，文件中包含多行，返回[]'''
    with open(filename, encoding='utf-8') as f:
        return f.readlines()


def read_txt(filename):
    '''读取单个txt文件的数据'''
    with open(filename, encoding='utf-8') as f:
        return f.read()


def read_json(filename):
    '''读取单个json类型文件，并load为dict'''
    with open(filename, mode='r', encoding='utf8') as f:
        return json.load(f)


def load_jsons(file_path):
    '''读取多个json类型文件，并装在到[] TODO'''
    D = []
    with open(file_path, mode='r', encoding='utf-8') as f:
        for l in f:
            l = json.loads(l)
            D.append(l)
    return D 


def copy_dir(src_path, target_path):
    '''拷贝文件夹到指定目录'''
    if os.path.isdir(src_path) and os.path.isdir(target_path):
        filelist_src = os.listdir(src_path)
        for file in filelist_src:
            path = os.path.join(os.path.abspath(src_path), file)
            if os.path.isdir(path):
                path1 = os.path.join(os.path.abspath(target_path), file)
                if not os.path.exists(path1):
                    os.mkdir(path1)
                copy_dir(path, path1)
            else:
                with open(path, 'rb') as read_stream:
                    contents = read_stream.read()
                    path1 = os.path.join(target_path, file)
                    with open(path1, 'wb') as write_stream:
                        write_stream.write(contents)
        return True

    else:
        return False

from pathlib import Path
from tempfile import NamedTemporaryFile 
def save_file(file,save_dir='./tmp'):
    try:
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)
        suffix = Path(file.filename).suffix

        with NamedTemporaryFile(delete=False, suffix=suffix, dir=save_dir) as tmp:
            shutil.copyfileobj(file.file, tmp)
            tmp_file_name = Path(tmp.name).name
        img_path = os.path.join(save_dir,tmp_file_name) 
        return  img_path
    finally:
        file.file.close()


'''
# 将PDF转化为图片
pdfPath pdf文件的路径
imgPath 图像要保存的文件夹
zoom_x x方向的缩放系数
zoom_y y方向的缩放系数
rotation_angle 旋转角度
TODO 性能较慢，需要优化
'''
@logtime
def pdf_image(pdfPath,imgPath,zoom_x = 5,zoom_y = 5,rotation_angle = 0):
    # 打开PDF文件
    if not os.path.exists(imgPath):
        os.mkdir(imgPath) 
        
    pdf = fitz.open(pdfPath)
    D = []
    # 逐页读取PDF
    for pg in range(0, pdf.pageCount):
        page = pdf[pg]
        # 设置缩放和旋转系数
        trans = fitz.Matrix(zoom_x, zoom_y).preRotate(rotation_angle)
        pm = page.getPixmap(matrix=trans, alpha=False)
        # 开始写图像
        f = os.path.join(imgPath,str(pg)+".png") 
        pm.writePNG(f) 
        D.append(f)
    pdf.close()
    
    return D

@logtime
def is_img_pdf(path):
    '''
    #  根据pdf中图片数量判断是否是双层PDF
    TODO 存在一定的错误性（折中方案）
    :param path: pdf的路径 
    :return:
    ''' 
    # 使用正则表达式来查找图片
    checkXO = r"/Type(?= */XObject)" 
    checkIM = r"/Subtype(?= */Image)"  
    
    # 打开pdf
    doc = fitz.open(path)
    # 图片计数
    imgcount = 0
    lenXREF = doc._getXrefLength()
    
    # 遍历每一个对象
    for i in range(1, lenXREF):
        # 定义对象字符串
        text = doc._getXrefString(i)
        isXObject = re.search(checkXO, text)
        # 使用正则表达式查看是否是图片
        isImage = re.search(checkIM, text)
 
        # 如果不是对象也不是图片，则continue
        if not isXObject or not isImage:
            continue
        imgcount += 1   
    
    print("文件名:{}, 页数: {}, 对象: {}, 图片:{}".format(path, len(doc), lenXREF - 1,imgcount))
    if len(doc) == imgcount:
        return True
    else :
        return False
    
def copy_dirs(src_path,target_path):
    '''拷贝指定文件夹中的文件到指定目录，没有目录结构'''
    file_count=0
    source_path = os.path.abspath(src_path)
    target_path = os.path.abspath(target_path)
    if not os.path.exists(target_path):
        os.makedirs(target_path)
    if os.path.exists(source_path):
        for root, dirs, files in os.walk(source_path):
            for file in files:
                src_file = os.path.join(root, file)
                shutil.copy(src_file, target_path)
                file_count+=1
                print(src_file)
    return int(file_count)